#!/bin/sh
###############################################
#        Patch script for SDSU v2.1.2
#                   Hiro Masunaga, May 21, 2010
###############################################

echo "This script upgrades the SDSU v2.* to v2.1.2."
echo "If you don't have the SDSU version 2 installed at all,"
echo "please cancel this script and download the whole package."
echo "Do you have an earlier version of SDSU-v2? (y/n)"
read yesno
if [ "$yesno" = "n" ]; then
     exit
fi
PTCHPATH=`pwd`
SDSUPATH=`cd ..; pwd`
echo "It is assumed that the path to your SDSU is "
echo $SDSUPATH
echo "Is this correct?(y/n)"
read yesno
if [ "$yesno" = "n" ]; then
     echo "Please type the FULL path to your SDSU."
     read SDSUPATH
fi
#
echo "Please type the version of your SDSU (2.0/2.1.0/2.1.1)."
echo "You can check your version with SDSU-v2/RELEASE_NOTES,"
echo "where the current version is indicated in the top line."
read version
#
# v2.1.1 -> v2.1.2
#
cd $PTCHPATH/src/simulator
MAIN=$SDSUPATH/src/simulator
BMCV=beamconv
VSIR=vis_IR
PLIN=plug-ins
#
if [ "$version" = "2.1.1" -o "$version" = "2.1.0" ]; then
echo "The SDSU v2.1.* is being upgraded to the v2.1.2."
echo "      ### User Interface Routines ###"
echo "For each routine, you will be asked whether the existing file"
echo "may be overwritten. If you answer no, a separate file"
echo "(*.f90_v2.1.2) will be created to store the copy."
#
for file in $BMCV/param_set_bmcv.f90 $BMCV/makefile def_file_list.f90 $PLIN/def_file_list_CReSS.f90
do
echo "Overwrite "$file"? (y/n)"
read yesno
if [ "$yesno" = "y" ]; then
     \cp $file $MAIN/$file
else
     \cp $file $MAIN/${file}_v2.1.2
fi
done
#
echo "       ### Internal Routines ###"
echo "Only internal routines that are usually not edited by users"
echo "will be installed and thus replaced without being backed up."
echo "If you don't want any of them overwritten, exit now and"
echo "back them up before you proceed further. See README for "
echo "the list of the files to be replaced. Ready to proceed?(y/n)"
read yesno
if [ "$yesno" = "n" ]; then
     exit
fi
#
cd $PTCHPATH
echo "updating the release notes."
\cp RN_patch2.1.2 $SDSUPATH/RELEASE_NOTES
cd $PTCHPATH/src/simulator
for file in $VSIR/rstr6b.f90 $BMCV/convolv_radar.f90
do
echo "copying "$file
\cp $file $MAIN/$file
done
#
fi
if [ "$version" = "2.1.1" ]; then
exit
fi
#
# v2.1.0 -> v2.1.2
#
if [ "$version" = "2.1.0" ]; then
#
MCRW=src/simulator/microw
RADR=src/simulator/radar
#
cd $PTCHPATH
file=SIMmicro.f90
echo "copying "$file
\cp $MCRW/$file $SDSUPATH/$MCRW/$file
file=SIMradar.f90
echo "copying "$file
\cp $RADR/$file $SDSUPATH/$RADR/$file
#
exit
fi
#
# v2.0 -> v2.1.2
#
echo "The SDSU v2.0 is being upgraded to the v2.1.2."
echo "--------- Main Simulator Units -----------"
cd $PTCHPATH/src/simulator
MAIN=$SDSUPATH/src/simulator
GASD=gas
HYDR=hydromet
MCRW=microw
RADR=radar
VSIR=vis_IR
BMCV=beamconv
PLIN
#
echo "      ### User Interface Routines ###"
echo "For each routine, you will be asked whether the existing file"
echo "may be overwritten. If you answer no, a separate file"
echo "(*.f90_v2.1.2) will be created to store the copy."
#
for file in param_set.f90 $HYDR/microp_set.f90 $BMCV/param_set_bmcv.f90 def_file_list.f90
do
echo "Overwrite "$file"? (y/n)"
read yesno
if [ "$yesno" = "y" ]; then
     \cp $file $MAIN/$file
else
     \cp $file $MAIN/${file}_v2.1.2
fi
done
#
echo "      ### Makefile, rd_CRM, and F90 Modules ###"
echo "For each file, you will be asked if you want a backup file created."
echo "If you answer yes, the user's module files will be duplicated as"
echo "*.f90_bkup before they are copied."
#
for file in makefile rd_CRM.f90 MAINparameters.f90 */*parameters.f90 $BMCV/makefile
do
echo "Back up "$file"? (y/n)"
read yesno
if [ "$yesno" = "y" ]; then
     \mv $MAIN/$file $MAIN/${file}_bkup
fi
\cp $file $MAIN/$file
done
#
echo "       ### Internal Routines ###"
echo "Other internal routines are usually not edited by users and"
echo "will be replaced without being backed up. If you don't want" 
echo "any of them overwritten, exit now and back them up before you"
echo "proceed further. See README for the list of the files to be replaced."
echo "Ready to proceed?(y/n)"
read yesno
if [ "$yesno" = "n" ]; then
     exit
fi
#
cd $MAIN
if [ ! -e beamconv ]; then
     ln -s ../beamconv beamconv
fi
#
cd $PTCHPATH/src/beamconv
$file=convolv_radar.f90
echo "copying "$file
\cp $file $SDSUPATH/src/beamconv/$file
#
cd $PTCHPATH/src/simulator
for file in dealloc_CRM.f90 
do
echo "copying "$file
\cp $file $MAIN/$file
done
for file in $GASD/absorb.f
do
echo "copying "$file
\cp $file $MAIN/$file
done
for file in $HYDR/heymsplatt.f90 $HYDR/mie_*.f90
do
echo "copying "$file
\cp $file $MAIN/$file
done
for file in $MCRW/SIMmicro.f90 $MCRW/*lut_mwr.f90 $MCRW/README-microw
do
echo "copying "$file
\cp $file $MAIN/$file
done
for file in $RADR/SIMradar.f90 $RADR/*lut_rdr.f90 $RADR/README-radar $RADR/radar_echo.f90
do
echo "copying "$file
\cp $file $MAIN/$file
done
for file in $VSIR/SIMvisir.f90 $VSIR/*lut_vir.f90 $VSIR/lib_*.f90 $VSIR/rstr6b.f90
do
echo "copying "$file
\cp $file $MAIN/$file
done
echo "copying plug-ins/"
\cp -r plug-ins $MAIN/
#
echo "--------- Mie LUT Generator -----------"
cd $PTCHPATH/src/MieLUT
MLUT=$SDSUPATH/src/MieLUT
#
echo "      ### User Interface Routines ###"
echo "For each routine, you will be asked whether the existing file"
echo "may be overwritten. If you answer no, a separate file"
echo "(*.f90_v2.1.2) will be created to store the copy."
#
for file in paramLUT_set.f90
do
echo "Overwrite "$file"? (y/n)"
read yesno
if [ "$yesno" = "y" ]; then
     \cp $file $MLUT/$file
else
     \cp $file $MLUT/${file}_v2.1.2
fi
done
#
echo "      ###  F90 Modules ###"
echo "For each file, you will be asked if you want a backup file created."
echo "If you answer yes, the user's module files will be duplicated as"
echo "*.f90_bkup before they are copied."
#
for file in LUTparameters.f90
do
echo "Back up "$file"? (y/n)"
read yesno
if [ "$yesno" = "y" ]; then
     \mv $MLUT/$file $MLUT/${file}_bkup
fi
\cp $file $MLUT/$file
done
#
echo "       ### Internal Routines ###"
echo "Other internal routines are usually not edited by users and"
echo "will be replaced without being backed up. If you don't want" 
echo "any of them overwritten, exit now and back them up before you"
echo "proceed further. See README for the list of the files to be replaced."
echo "Ready to proceed?(y/n)"
read yesno
if [ "$yesno" = "n" ]; then
     exit
fi
#
for file in mktab_*f90
do
echo "copying "$file
\cp $file $MLUT/$file
done
#
echo "--------- Mie LUT -----------"
cd $SDSUPATH/Mie-table
if [ ! -e exp2mo ]; then
     mkdir exp2mo
fi
mkdir $SDSUPATH/Mie-table/lgn2mo
if [ ! -e lgn2mo ]; then
     mkdir lgn2mo
fi
#
cd $PTCHPATH/Mie-table/melt-p
MLUT=$SDSUPATH/Mie-table/melt-p
echo "       ### Mie LUT for melting particles ###"
echo "The Mie lookup tables for melting particles have been updated."
echo "The default LUTs in Mie-table/melt-p will be replaced. "
echo "If you have created your own LUT, you may need to run the"
echo "Mie LUT generator again before the main simulator is used."
echo "Ready to proceed?(y/n)"
read yesno
if [ "$yesno" = "n" ]; then
     exit
fi
\cp * $MLUT/.
#
