! ############################################################
!            Satellite Data Simulation Unit v2
!                     - open_lut_rdr -
!
! This subroutine reads the Mie lookup tables created by the
! program stored in "src/MieLUT". Any changes made to the LUT
! format must be reflected in this subroutine. Make sure
! that the specified file names match existing LUTs. The path
! to LUTs is given by "lutdir" defined in 'MAINparameters'.
!
!             Updated from open-lut-rdr.f by H. M. (May, 2009)
!                Updated for SDSU v.2.1.0 by H. M. (Aug, 2009)
!
 Subroutine open_lut_rdr ( nf )
   Use MAINparameters
   Use RDRparameters
   Use MCRPparameters
   Implicit NONE
!
! I/O parameters
!
   integer, intent(in) :: nf
!
! Local variables
!
   real :: frequency
   Integer :: i,j,k,npts_ncnc1,npts_ncnc2
   Character*4  :: nch, mklab_lut
   Character*50 :: fname_in
   Character*100 :: pathandfile
! 
! "nch" is automatically assigned by "mklab_lut"
!
   nch = mklab_lut ( freq_radar(nf) )
!
! Cloud Ice
!
   fname_in = cldi%functype//'/'//trim(cldi%name)//'-rdr'//nch//'.dat'
   pathandfile = lutdir//fname_in
   Open ( Unit=50, File=pathandfile )
!
   read(50,*) npts_temp, npts_hydro, npts_ncnc
!
   allocate ( kextci_lut(npts_hydro,npts_temp,npts_ncnc) )
   allocate ( salbci_lut(npts_hydro,npts_temp,npts_ncnc) )
   allocate ( pbckci_lut(npts_hydro,npts_temp,npts_ncnc) )
   allocate ( ptn_cldi(npts_ncnc), pts_hydro(npts_hydro) )
   allocate ( pts_temp_i(npts_temp) )
!
   read(50,*) (pts_temp_i(i),i=1,npts_temp)
   read(50,*) (pts_hydro(i),i=1,npts_hydro)
   read(50,*) (ptn_cldi(i) ,i=1,npts_ncnc )
   read(50,*) frequency
!
   read(50,*) (((kextci_lut(i,j,k),salbci_lut(i,j,k),pbckci_lut(i,j,k), &
                     i = 1,npts_hydro),j = 1,npts_temp),k=1,npts_ncnc)
   close (50)
!
! Cloud Water
!
   fname_in = cldw%functype//'/'//trim(cldw%name)//'-rdr'//nch//'.dat'
   pathandfile = lutdir//fname_in
   Open ( Unit=50, File=pathandfile )
!
   read(50,*) npts_temp, npts_hydro, npts_ncnc
!
   allocate ( kextcw_lut(npts_hydro,npts_temp,npts_ncnc) )
   allocate ( salbcw_lut(npts_hydro,npts_temp,npts_ncnc) )
   allocate ( pbckcw_lut(npts_hydro,npts_temp,npts_ncnc) )
   allocate ( pts_temp_w(npts_temp), ptn_cldw(npts_ncnc) )
!
   read(50,*) (pts_temp_w(i),i=1,npts_temp)
   read(50,*) (pts_hydro(i),i=1,npts_hydro)
   read(50,*) (ptn_cldw(i) ,i=1,npts_ncnc )
   read(50,*) frequency
!
   read(50,*) (((kextcw_lut(i,j,k),salbcw_lut(i,j,k),pbckcw_lut(i,j,k), &
                     i = 1,npts_hydro),j = 1,npts_temp),k=1,npts_ncnc)
   close (50)
!
! Rain
!
   fname_in = rain%functype//'/'//trim(rain%name)//'-rdr'//nch//'.dat'
   pathandfile = lutdir//fname_in
   Open ( Unit=50, File=pathandfile )
!
   read(50,*) npts_temp, npts_hydro, npts_ncnc
!
   allocate ( kextrr_lut(npts_hydro,npts_temp,npts_ncnc) )
   allocate ( salbrr_lut(npts_hydro,npts_temp,npts_ncnc) )
   allocate ( pbckrr_lut(npts_hydro,npts_temp,npts_ncnc) )
   allocate ( ptn_rain(npts_ncnc) )
!
   read(50,*) (pts_temp_w(i),i=1,npts_temp)
   read(50,*) (pts_hydro(i),i=1,npts_hydro)
   read(50,*) (ptn_rain(i) ,i=1,npts_ncnc )
   read(50,*) frequency
!
   read(50,*) (((kextrr_lut(i,j,k),salbrr_lut(i,j,k),pbckrr_lut(i,j,k), &
                     i = 1,npts_hydro),j = 1,npts_temp),k=1,npts_ncnc)
   close (50)
!
! Snow
!
   fname_in = snow%functype//'/'//trim(snow%name)//'-rdr'//nch//'.dat'
   pathandfile = lutdir//fname_in
   Open ( Unit=50, File=pathandfile )
!
   read(50,*) npts_temp, npts_hydro, npts_ncnc
!
   allocate ( kextsn_lut(npts_hydro,npts_temp,npts_ncnc) )
   allocate ( salbsn_lut(npts_hydro,npts_temp,npts_ncnc) )
   allocate ( pbcksn_lut(npts_hydro,npts_temp,npts_ncnc) )
   allocate ( ptn_snow(npts_ncnc) )
!
   read(50,*) (pts_temp_i(i),i=1,npts_temp)
   read(50,*) (pts_hydro(i),i=1,npts_hydro)
   read(50,*) (ptn_snow(i) ,i=1,npts_ncnc )
   read(50,*) frequency
!
   read(50,*) (((kextsn_lut(i,j,k),salbsn_lut(i,j,k),pbcksn_lut(i,j,k), &
                     i = 1,npts_hydro),j = 1,npts_temp),k=1,npts_ncnc)
   close (50)
!
! Graupel
!
   fname_in = grpl%functype//'/'//trim(grpl%name)//'-rdr'//nch//'.dat'
   pathandfile = lutdir//fname_in
   Open ( Unit=50, File=pathandfile )
!
   read(50,*) npts_temp, npts_hydro, npts_ncnc
!
   allocate ( kextgr_lut(npts_hydro,npts_temp,npts_ncnc) )
   allocate ( salbgr_lut(npts_hydro,npts_temp,npts_ncnc) )
   allocate ( pbckgr_lut(npts_hydro,npts_temp,npts_ncnc) )
   allocate ( ptn_grpl(npts_ncnc) )
!
   read(50,*) (pts_temp_i(i),i=1,npts_temp)
   read(50,*) (pts_hydro(i),i=1,npts_hydro)
   read(50,*) (ptn_grpl(i) ,i=1,npts_ncnc )
   read(50,*) frequency
!
   read(50,*) (((kextgr_lut(i,j,k),salbgr_lut(i,j,k),pbckgr_lut(i,j,k), &
                     i = 1,npts_hydro),j = 1,npts_temp),k=1,npts_ncnc)
   close (50)
!
! Hail
!
   fname_in = hail%functype//'/'//trim(hail%name)//'-rdr'//nch//'.dat'
   pathandfile = lutdir//fname_in
   Open ( Unit=50, File=pathandfile )
!
   read(50,*) npts_temp, npts_hydro, npts_ncnc
!
   allocate ( kexthl_lut(npts_hydro,npts_temp,npts_ncnc) )
   allocate ( salbhl_lut(npts_hydro,npts_temp,npts_ncnc) )
   allocate ( pbckhl_lut(npts_hydro,npts_temp,npts_ncnc) )
   allocate ( ptn_hail(npts_ncnc) )
!
   read(50,*) (pts_temp_i(i),i=1,npts_temp)
   read(50,*) (pts_hydro(i),i=1,npts_hydro)
   read(50,*) (ptn_hail(i) ,i=1,npts_ncnc )
   read(50,*) frequency
!
   read(50,*) (((kexthl_lut(i,j,k),salbhl_lut(i,j,k),pbckhl_lut(i,j,k), &
                     i = 1,npts_hydro),j = 1,npts_temp),k=1,npts_ncnc)
   close (50)
!
! Melting particles
!
   If ( meltp ) then
!
   fname_in = 'melt-p/'//snow%functype//trim(snow%name)//'_' & 
                       //rain%functype//trim(rain%name)//'-rdr'//nch//'.dat'
   pathandfile = lutdir//fname_in
   Open ( Unit=50, File=pathandfile )
!
   read(50,*) npts_hydro, npts_ncnc1, npts_ncnc2
!
   allocate ( kextml_lut(npts_hydro,npts_ncnc1,npts_ncnc2) )
   allocate ( salbml_lut(npts_hydro,npts_ncnc1,npts_ncnc2) )
   allocate ( pbckml_lut(npts_hydro,npts_ncnc1,npts_ncnc2) )
!
   read(50,*) (pts_hydro(i),i=1,npts_hydro)
   read(50,*) (ptn_rain(i) ,i=1,npts_ncnc1)
   read(50,*) (ptn_snow(i) ,i=1,npts_ncnc2)
   read(50,*) frequency
!
   read(50,*) (((kextml_lut(i,j,k),salbml_lut(i,j,k),pbckml_lut(i,j,k), &
                       i = 1,npts_hydro),j=1,npts_ncnc1),k=1,npts_ncnc2)
   close (50)
!
   End If
!
   Return
 End Subroutine open_lut_rdr

