! ############################################################
!            Satellite Data Simulation Unit v2
!                     - mie_lut_vir -
!
! This subroutine find the extinction/scattering coefficients and
! phase function for given hydrometeor water content and number 
! concentration using the Mie LUTs. 
!
!                  Updated from mie-lut.f by H. M. (May, 2009)
!                Updated for SDSU v.2.1.0 by H. M. (Aug, 2009)
!
 Subroutine mie_lut_vir(Whydr,x1,x2,pts_x1,pts_x2, &
                        cext_lut,csca_lut,phsf_lut, &
                        cext8,csca8,phsf8)
   Use VIRparameters
   Implicit NONE
!
! I/O parameters
! 
   real(8), intent(in)  :: Whydr,x1,x2
   real, dimension(:), intent(in) :: pts_x1,pts_x2
   real, dimension(:,:,:), intent(in) :: cext_lut,csca_lut
   real, dimension(:,:,:,:), intent(in) :: phsf_lut
   real(8), intent(out) :: cext8,csca8
   real(8), dimension(:), intent(out) :: phsf8
!
! Local variables
!
   real, dimension(:,:,:), allocatable :: phsf2_lut
   real, dimension(:,:), allocatable :: hydr2_lut,phsf1_lut
   real, dimension(:), allocatable :: hydr1_lut
   integer :: nph,nx1,nx2
   real :: f1,fW,f2
   real :: cext,csca
   real, dimension(knang) :: phsf
   Integer :: i,j,k
!
   cext = 0.d0 ; csca = 0.d0 ; phsf = 0.d0
   nph = size(cext_lut,dim=1)
   nx1 = size(cext_lut,dim=2)
   nx2 = size(cext_lut,dim=3)
   allocate ( hydr2_lut(nph,nx1), hydr1_lut(nph) )
   allocate ( phsf2_lut(knang,nph,nx1), phsf1_lut(knang,nph) )
!
! Skip if x2 is out of range except when nx2 = 1.
!
   If (nx2 > 1 .and. x2 < pts_x2(1))   Return
   If (nx2 > 1 .and. x2 > pts_x2(nx2)) Return
!
! Skip if x1 is out of range except where nx1 = 1.
!
   If (nx1 > 1 .and. x1  < pts_x1(1))    Return
   If (nx1 > 1 .and. x1  > pts_x1(nx1))  Return
!      
! Skip if W is out of range.
!
   If (Whydr < pts_hydro(1))   Return
   If (Whydr > pts_hydro(nph)) Return
!
! /////////////// cext //////////////////
!
! Locate x2
!
   Do k = 2, nx2
      If (x2 > pts_x2(k)) cycle
      f2 = (pts_x2(k)-x2) / (pts_x2(k)-pts_x2(k-1))
      hydr2_lut = f2*cext_lut(:,:,k-1) + (1.-f2)*cext_lut(:,:,k)
      exit
   End do
   If ( nx2 == 1 )  hydr2_lut = cext_lut(:,:,1)
!
! Locate x1
!
   Do j = 2 , nx1
      If (x1 > pts_x1(j)) cycle
      f1 = (pts_x1(j)-x1) / (pts_x1(j)-pts_x1(j-1))
      hydr1_lut = f1*hydr2_lut(:,j-1) + (1.-f1)*hydr2_lut(:,j)
      exit
   End do
   If ( nx1 == 1 ) hydr1_lut = hydr2_lut(:,1)
!
! Locate W
!
   Do i = 2 , nph
      If (Whydr > pts_hydro(i)) cycle
      fW = (pts_hydro(i)-Whydr) / (pts_hydro(i)-pts_hydro(i-1))
      cext = fW*hydr1_lut(i-1) + (1.-fW)*hydr1_lut(i)
      exit
   End do
!
! /////////////// csca //////////////////
!
! Locate x2
!
   Do k = 2, nx2
      If (x2 > pts_x2(k)) cycle
      f2 = (pts_x2(k)-x2) / (pts_x2(k)-pts_x2(k-1))
      hydr2_lut = f2*csca_lut(:,:,k-1) + (1.-f2)*csca_lut(:,:,k)
      exit
   End do
   If ( nx2 == 1 )  hydr2_lut = csca_lut(:,:,1)
!
! Locate x1
!
   Do j = 2 , nx1
      If (x1 > pts_x1(j)) cycle
      f1 = (pts_x1(j)-x1) / (pts_x1(j)-pts_x1(j-1))
      hydr1_lut = f1*hydr2_lut(:,j-1) + (1.-f1)*hydr2_lut(:,j)
      exit
   End do
   If ( nx1 == 1 ) hydr1_lut = hydr2_lut(:,1)
!
! Locate W
!
   Do i = 2 , nph
      If (Whydr > pts_hydro(i)) cycle
      fW = (pts_hydro(i)-Whydr) / (pts_hydro(i)-pts_hydro(i-1))
      csca = fW*hydr1_lut(i-1) + (1.-fW)*hydr1_lut(i)
      exit
   End do
!
! /////////////// phsf //////////////////
!
! Locate x2
!
   Do k = 2 , nx2
      If (x2 > pts_x2(k)) cycle
      f2 = (pts_x2(k)-x2) / (pts_x2(k)-pts_x2(k-1))
      phsf2_lut(1:nang,:,:) = f2*phsf_lut(1:nang,:,:,k-1) + &
                         (1.-f2)*phsf_lut(1:nang,:,:,k)
      exit
   End do
   If ( nx2 == 1 ) phsf2_lut(1:nang,:,:) = phsf_lut(1:nang,:,:,1)
!
! Locate x1
!
   Do j = 2 , nx1
      If (x1 > pts_x1(j)) cycle
      f1 = (pts_x1(j)-x1) / (pts_x1(j)-pts_x1(j-1))
      phsf1_lut(1:nang,:) = f1*phsf2_lut(1:nang,:,j-1) + &
                       (1.-f1)*phsf2_lut(1:nang,:,j)
      exit
   End do
   If ( nx1 == 1 ) phsf1_lut(1:nang,:) = phsf2_lut(1:nang,:,1)
!
! Locate W
!
   Do i = 2 , nph
      If (Whydr > pts_hydro(i)) cycle
      fW = (pts_hydro(i)-Whydr) / (pts_hydro(i)-pts_hydro(i-1))
      phsf(1:nang) = fW*phsf1_lut(1:nang,i-1) + (1.-fW)*phsf1_lut(1:nang,i)
      exit
   End do
!
   cext8 = cext ; csca8 = csca ; phsf8(1:nang) = phsf(1:nang)
   deallocate ( hydr2_lut, hydr1_lut, phsf2_lut, phsf1_lut )
   Return
 End Subroutine mie_lut_vir
