subroutine rstr6b(init,iuk,iug,indg,inda,imthd,isol,nda,na0,th0, &
     na1,th1,nfi,fi,wl,dw,nw0,rx,rf,galb,icn,wlcn,npoly,ncomp,mptc, &
     vptc,cnpt,nl,alt,prs,tmp,nmol,cng,nptc,cnp,ispcvp,rfracp,asphr, &
     rop,dryap,nawp,awcrp,nv,nwlv,wlv,rfi,nln,ipbf,nch,dintvl,stype, &
     thk0,sol,flxd0,flxd00,flxu0,ai0,err)
!
!                 Updated for SDSUv2.1.0  by H. M. (Aug, 2009)
!                       with mie_lut_vir thoroughly rewritten.
!
! general subroutine for calculating radiation field
!--- history
! 95. 9.15 created
! 95.12.28 corrected the comments for icn=1, and added icn=3 case.
! 95.12.30 modified the relative humidity profile calculation
! 96. 3.11 added non-spherical parameters
!     5. 5 introduce ncomp and some bug-fix for cnprf1
!     5. 9 cextp() -> cextp2, cabsp() -> cabsp2, voldp()->voldp2, volp()->volp2
!     5.24 change the structure of do 18
! 96.12.27 bug in initialization with do 17
! 96.12.28 change the meaning of rx
! 97. 1.14  20.0->200.0
! 97. 3.17  get scr and sci from subroutine rfidxb.
!           call rtrn21 with scr and sci.
! 98.1.7
!   u10 > 0.01 for rstr and pstr.  u10 < 0.01 will have significant
!   error in radiances due to angular integration errors for surface
!   reflection.  recommend that u10 is set to be 0.01 for flat surface
!   case
! 06.06.25 version up to 6. (miho)
! 07.03.28 fixed some bugs. (miho)
! 07.11.30 dynamical => statical dimension (nln=>knln, nch=>kch) (miho) 
! 08. 2.19 flxd00 for direct solar flux
! 08. 2.19 added notes for thk0
! 08. 2.19 deleted ifrh and trh
! 08. 8.14 rtrn2 --> rtrn21, add dintvl
! 08. 9.19 modified free-style form
!--- input
! init       i        if 1 initilize the routine
!                     0 skip wavelength-independent part
! iuk        i        device number for a kernel file
! indg       i       -1: no ground surface
!                     0: lambert surface
!                     1: ocean surface initialization
!                     2: ocean surface with no initialization
!                     when indg>0 and imthd>0 then cabspsingle scattering correction
!                      for ocean surface reflection
! inda       i        0: flux; 1: radiance
! imthd      i       -1: nt,  0: dms-method  for intensity/flux
!                     1: ms,  2:tms,  3:ims-method for intensity.
!                     when indg>0 and imthd>0 then single scattering correction
!                      for ocean surface reflection
! isol       i        0: night
!                     1: day
! nda        i        nunber of nadir-quadrature angles in the
!                     hemisphere.
! na0        i        number of solar incidences.
! th0     r(na0)      solar zenith angles 0-90 degrees
! na1        i        number of emergent nadir angles in sphere.
! th1    r(kna1u)     emergent nadir angles.
!                      0-90 for downward; 90-180 for upward
! nfi        i        number of azimuthal angles.
! fi     r(knfi)      azimuthal angles in degrees.
! wl         r        center wavelengths (micron).
! dw         r        scaling factor to define scaled wavelengths rx.
!                     give 0 for monochromati! calculations.
! nw0        r        number of sub-wavelength grids for calculating response
!                      function.
! rx     r(knw0+1)    dw*rx(i) = boundaries in micron for sub-wavelenth integration
!                     of the response function rf.  -0.5 <= rx <= 0.5
! rf      r(knw0)     integrated response function of radiometer over
!                       wavelength interval of (rx(i)*dw, rx(i+1)*dw)
!                     averaged radiance is given as
!                       l = sum (i=1,n; radiance(i)*rf(i))/sum(i=1,n; rf(i))
!                     give nw0=1, rx(1)=rx(2)=0, rf(1)=1 for monochromatic
!                     calculations.
! galb       r        ground albedo if indg=0
!                     u10 (m/sec)   if indg>0; u10>0.01
! icn        i        0: cnpt is the total column dry volume for each polydispersion
!                     1: cnpt is the total column volume for each polydispersion
!                     2: cnpt is optical thickness at wlcn
!                     3: cnpt is optical thickness at each wavelength.
! wlcn       r        scaling wavelength for icn=2,3
! npoly      i        number of external mixtures
! ncomp   i(npoly)    number of aerosol types for the external mixture
! mptc    i(kpoly,4)  aerosol types for the external mixture
! vptc    r(kpoly,4)  dry volume ratio for the mixture
! cnpt    r(npoly)    total volume (icn=0,1) or optical thickness (icn=2,3)
!                      for each polydispersion.
! nl         i        number of atmospheric layers to define model atmosphere
! alt     r(knl)      height at interfaces of layers (km), top to bottom
! prs     r(knl)      pressure    at interfaces of layers (mb)
! tmp     r(knl)      temperature at interfaces of layers (k)
! nmol       i        number of gases
! cng    r(knl,knm0)  gas concentration (ppmv)
! nptc       i        number of defined aerosol types
! cnp    r(knl,kptc)  dry volume concentration profile (relative unit)
! ispcvp i(3,kptc)    fundamental materials for 3 comp. internal mixture (1-8)
! rfracp r(3,kptc)    dry volume fraction of the dry mixture
! asphr  r(3,kptc)    non-spherical parameters (x0, g, r)
! rop    r(kptc)      paricle density relative to water
! dryap  r(6,4,kptc)  dv/dlnr parameters for the dry mixture
!                     see vlspc2
!                     c-values (coefficients of volume spectrum) are ralative
! nawp   i(kptc)      number of aw
! awcrp r(kaw,kptc,2) 1: aw      water activity (see shettle and fenn.)
!                     2: rmmd    rmmd
! nv         i        number of fundamental species (1-8)
! nwlv       i        number of wavelengths for refractive index tables
! wlv    r(kwl)       wavelengths (micron)
! rfi   r(kwl,knv,2)  refractive index of fundamental materials (mr, mi)
!                     =mr - i mi
!                     with log-regular wavelength interval
! nln        i        number of atmospheric layers for radiative transfer.
! ipbf    i(nln1)     interface number to define sublayers to construct
!                      the transfer atmosphere in mlatm.
!                     ipbf(1)=50, ipbf(nln+1)=1
!                     if >0 then field calculations
!                        <0 then no field calculations
!--- output
! thk0    r(nln,10)   (l,1): total optical thickness for extinction (tau)
!                     (l,2): particle optical thickness for extinction (taua)
!                     (l,3): optical thickness for rayleigh scattering (taur)
!                     (l,4): single scattering albedo (w)
!                     (l,5): g1 (1st legendre polynomial moment)
!                     (l,6): g2 (2nd legendre polynomial moment) 
!   these values are not for monochromatic but for wavelength-mean values
!   other important parameters can be obtained as follows:
!   optical thickness for gas absorption: taug= tau - taua - taur
!   single scattering albedo of particles: wa = (w*tau-taur)/taua
!   legendre poly. moments of particles: gan= (w*tau*gn - grn*taur)/(wa*taua)
!    because  tau= taua + taur + taug
!             w*tau = wa*taua + taur
!             w*tau*gn = wa*taua*gan + grn*taur
!    where gr is the phase function of rayleigh scattering phase function
!   good approximation is: gr1= 0 and gr2= 0.1
! sol        r        solar incident irradiance (w/m2/micron)
! flxd0 r(kna0,kntau) downward flux (w/m2/micron)
! flxd00 r(kna0,kntau) downward direct flux (horizontal plane flux)
! flxu0 r(kna0,kntau) upward flux (w/m2/micron)
! ai0   r(kna1u,kna0,knfi,kntau)
!                     radiance (w/m2/micron/str)
! err       c*64      error code.  if ' ' then normal termination.
!---
  use MAINparameters, only: lut
  use VIRparameters
  implicit none
!
   Interface
      Subroutine mie_lut_vir ( Whydr,x1,x2,pts_x1,pts_x2, &
                               y1lut,y2lut,y3lut,y1,y2,y3 )
        Real(8), intent(in) :: x1,x2,Whydr
        Real,    intent(in) :: pts_x1(:),pts_x2(:)
        Real,    intent(in) :: y1lut(:,:,:),y2lut(:,:,:),y3lut(:,:,:,:)
        Real(8), intent(out) :: y1,y2,y3(:)
      End Subroutine mie_lut_vir
   End Interface
!
! input
  integer,intent(inout):: init   !! initialization flag for RSTR6B
  integer,intent(in):: iuk       !! number of kernel file 
  integer,intent(in):: iug       !! number of kernel file 

  integer,intent(inout):: indg   !! surface flag (0:Lambert, 1: Ocean)
  integer,intent(inout):: inda   !! radiance flag (0: flux only, 1: flux & radiance)
  integer,intent(in):: imthd     !! flux/intensity calculation modes
  integer,intent(in):: isol      !! solar incident flag (0:night, 1:day)
  integer,intent(in):: nda       !! number of stream

  integer,intent(in):: na0           !! number of solar zenith angles
  real(8),intent(in):: th0(kna0)     !! solar zenith angles (degree)

  integer,intent(in):: na1           !! number of zenith angles of emerging radiance
  real(8),intent(in):: th1(kna1u)    !! zenith angles of emerging radiances (degree)

  integer,intent(in):: nfi           !! azimuthal angles
  real(8),intent(in):: fi(knfi)      !! zenith angles (degree)

  real(8),intent(in):: wl        !! center wavelength (micron)
  real(8),intent(in):: dw        !! scaling factor to define scaled wavelengths

  integer,intent(in):: nw0           !! number of sub-wavelength grids
  real(8),intent(in):: rx(knw01)     !! boundaries for sub-wavelength integration
  real(8),intent(in):: rf(knw0)      !! integrated response function

  real(8),intent(in):: galb      !! ground albedo (indg=0)/ U10 [m/sec] (indg=1)

  integer,intent(in):: icn           !! unit of cnpt indicator
  real(8),intent(in):: wlcn          !! scaling wavelength [cm] (icn=2 or 3)
  integer,intent(in):: npoly         !! number of particle polydispersions

  integer,intent(in):: ncomp(kpoly)  !! number of particle models
  integer,intent(in):: mptc(kpoly,4) !! particle model number
  real(8),intent(in):: vptc(kpoly,4) !! relative dry volume concentration for mixing
  real(8),intent(in):: cnpt(kpoly)   !! column total quantity

  integer,intent(in):: nl            !! number of atmospheric layer 
  real(8),intent(in):: alt(knl)      !! altitude [km]
  real(8),intent(in):: prs(knl)      !! pressure [mb]
  real(8),intent(in):: tmp(knl)      !! temperature [K]
  integer,intent(in):: nmol          !! number of gases
  real(8),intent(in):: cng(knl,knm0) !! gas concentration [ppmv]
  integer,intent(in):: nptc          !! number of particle polydispersions
  real(8),intent(in):: cnp(knl,kptc) !! dry volume concentration
  integer,intent(in):: ispcvp(3,kptc) !! internal mixture of fundamental materials
  real(8),intent(in):: rfracp(3,kptc) !! dry component volume fractions
  real(8),intent(in):: asphr(3,kptc)  !! non-spherical parameters
  real(8),intent(in):: rop(kptc)     !! particle density of dry mixture [g/cm3]
  real(8),intent(in):: dryap(6,4,kptc) !! parameters to define volume size distribution
  integer,intent(in):: nawp(kptc)    !! number of RH to define particle growth
  real(8),intent(in):: awcrp(kaw,kptc,2) !! growth parameters

  integer,intent(in):: nv            !! number of fundamental species
  integer,intent(in):: nwlv          !! number of wavelenggths for rfi tables
  real(8),intent(in):: wlv(kwlv)     !! wavelengths [micron] for rfi tables
  real(8),intent(in):: rfi(kwlv,knv,2)  !! refractive index of fundamental materials

  integer,intent(in):: nln           !! number of sublayers
  integer,intent(in):: ipbf(knln1)   !! interface number to define sublayers

  integer,intent(in):: nch           !! number of channel (k-dist file)
  integer,intent(in):: dintvl        !! inverse of interval (k-dist file)
  integer,intent(in):: stype         !! surface type

! output

  real(8),intent(out):: thk0(knln,10) !! calculated optical parameters
  real(8),intent(out):: sol           !! solar incident irradiance [W/m2/micron]
  real(8),intent(out):: flxd0(kna0,kntau) !! downward flux [W/m2/micron]
  real(8),intent(out):: flxu0(kna0,kntau) !! upward flux [W/m2/micron]
  real(8),intent(out):: flxd00(kna0,kntau) !! downward direct flux [W/m2/micron]
  real(8),intent(out):: ai0(kna1u,kna0,knfi,kntau) !! radiance [W/m2/micron/str]
  character,intent(inout):: err*64    !! error code

! for wvcal
  integer:: ind                  !! indicator for output
  real(8):: ppmv,gm3,e,gm3s,es   !! quantities of water vapor

! for rfidxb
  real(8):: scr                  !! relative reflactive index (real part)
  real(8):: sci                  !! relative reflactive index (imaginary part)
  real(8):: voldp2               !! total dry volume of aerosol
  real(8):: volp2                !! total volume of aerosol
  real(8):: cextp2               !! extinction cross section
  real(8):: cabsp2               !! absorption cross section
  real(8):: php(knang)           !! volume scattering phase function

! for sdp2
  real(8),save:: dp(knln)             !! weights in pressure grids
  integer,save:: np(knln)             !! numbers in pressure grids

! for abgask
  integer:: nww                  !! number of integration
  real(8):: wlb                  !! longer wavelength boundary
  real(8),save:: amtpb(kmol1,knln)    !! gas concentration
  real(8),save:: tl(knln)             !! temperature in each layer
  real(8):: tkd(knln,kch,kww)    !! optical thickness of gas absorption

! for qgausn
  real(8):: gwt(kch)             !! Gaussian weights 
  real(8):: gmu(kch)             !! Gaussian points

! for rtrn21
  integer,save:: indt                 !! flag of optical depth (1:utau, 2:dpt)
  integer,save:: indp                 !! flag of phase function
  real(8):: am1u(kna1u)          !! cosine of the emergent nadir angles
  real(8):: am0(kna0)            !! cosine of solar zenith angles
  real(8):: thk(knln)            !! optical thickness of sublayers
  real(8):: omg(knln)            !! single scattering albedo
  integer:: nlgn1(knln)          !! maximum order of moments + 1 (indp<=0;imthd=3)
  real(8):: g(klgn1,knln)        !! Legendre moments of phase function (indp<=0)
  real(8):: phsf(knang,knln)     !! phase function (indp=1)
  real(8),save:: epsp                 !! truncation criterion of phase function moments
  real(8),save:: epsu                 !! convergence criterion of intensity (inda>0)
  real(8):: fsol                 !! solar irradiance at the system top
  integer:: nplk1                !! number of order to approximate plank + 1
  real(8):: cplk(kplk1,knln)     !! coefficient of planck function 
  real(8):: bgnd                 !! planck function at ground 
  integer,save:: ntau                 !! number of user defined optical depth
  real(8):: utau(kntau)          !! optical depths where the field is calculated
  real(8):: flxd(kna0,kntau)     !! downward flux
  real(8):: flxu(kna0,kntau)     !! upward flux
  real(8):: ai(kna1u,kna0,knfi,kntau) !! intensity

! for initialization
  integer:: ipoly,j,ll,l1
  integer:: m,initrf
  real(8):: rhl,pp,tt,p1,t1,p2,t2,rh1,rh2
  real(8),save:: cnml(kpoly,4),vptc1(kpoly,4),cnprf1(knln,kpoly,4)
  real(8):: dz, p,t
! / added for SDSU /
  real(8), dimension(knln) :: dzl
  real(8),save:: volt(kpoly),cextpn(kpoly)
! / added for SDSU v2.1.2 /
  real(8) :: RT0_p0g
! for work
  integer:: iws,l,iww,ik,lf

  integer,save:: nln1                 !! nln + 1
  integer,save:: ipb(knln1)           !! abs(ipbf)
  real(8),save:: gtmp                 !! ground temperature
  real(8):: pl(knln)             !! pressure in each layer
  real(8),save:: rh(knln)             !! relative humidity in each layer
  real(8),save:: phsm(knang)          !! Rayleigh phase function

  real(8):: wla,wl1,wgt1,wlc,wn  !! wavelenghs
  real(8):: sunir                !! function of solar irradiance

  real(8):: tray,taur(knln)          !! Rayleigh
  real(8):: taua(knln),scaa(knln)    !! aerosol scattering
  real(8):: taua3(knln),scaa3(knln),phsf3(knang,knln)
  real(8):: cext1,cabs1,csca1
  real(8):: fac
  real(8):: wgt(kch)

  real(8):: taumax
  real(8):: dpt
  real(8):: bplnk,b1,b2

  real(8):: wgt2,wg0,sca2(knln)
  real(8):: thks
! / added for SDSU /
  real(8) :: Tavg
  real(8),dimension(knln) :: Wcw,Wci,Wrr,Wsn,Wgr,Whl
  real(8),dimension(knln) :: Ncw,Nci,Nrr,Nsn,Ngr,Nhl
  real(8),dimension(knln)       :: cextcw,cextci,cextrr,cextsn,cextgr,cexthl
  real(8),dimension(knln)       :: cscacw,cscaci,cscarr,cscasn,cscagr,cscahl
  real(8),dimension(knang,knln) :: dscscw,dscsci,dscsrr,dscssn,dscsgr,dscshl
  real(8),dimension(knang,knln) :: dscs
  complex(kind(0d0)) :: crefd
!
! / added for SDSU v2.1.2 /
!
  RT0_p0g = 8.31451d0*273.15d0/(1013.25d0*9.8d0*28.8d0)
!
!--exec
  if(init==1) then
     init=0
     pi=4.d0*atan(1.d0)
     rad=pi/180.d0
     if(inda.ge.1) inda   = 1
     indp = 1
     epsp = 0.d0; epsu = 0.d0
     nln1=nln+1

! level checking
     ipb(1:nln1)=abs(ipbf(1:nln1))
     ntau=count(ipbf(1:nln1)>0)

! 96.12.27 added T. Nakajima
     if(ntau<=0) then
        err='no interface for calculating radiation field'; return
     endif
! 96.12.27 end

     if(ntau>kntau) then
        err='number of field calculation layers larger than kntau'; return
     endif

     if(ntau==nln1) then
        indt=2
     else
        indt=1
     endif

! level merging (from top to bottom)
! / removed for SDSU /
!     gtmp=tmp(1)
!
!     do ipoly=1,npoly
!        do j=1,ncomp(ipoly)
!           cnml(ipoly,1:ncomp(j))=0.d0
!           cnprf1(1:nln,ipoly,1:ncomp(j))=0.d0
!
! 090710 Bug fixed (miho) 1:ncomp(j) --> 1:ncomp(ipoly)
!           vptc1(ipoly,j)=vptc(ipoly,j)/sum(vptc(ipoly,1:ncomp(ipoly)))
!        enddo
!     enddo
!
     amtpb(1:kmol,1:nln)=0.d0
     do ll = 1 ,nln
        rhl=0.d0; pp=0.d0; tt=0.d0

!! merging
        do l=ipb(ll+1),ipb(ll)-1
           l1=l+1
! / removed for SDSU /
!           dz=abs(alt(l)-alt(l1)) ; dzcm=dz*1.0d5
!           p =(prs(l)+prs(l1))*0.5d0
!           t =(tmp(l)+tmp(l1))*0.5d0
! / added for SDSU /
!           dz=abs(hgt_sdsu(l)-hgt_sdsu(l1))
           p =(prs_sdsu(l)+prs_sdsu(l1))*0.5d0
           t =(tmp_sdsu(l)+tmp_sdsu(l1))*0.5d0
!! 06.03.17 miho
           pp=pp+p; tt=tt+t
! / once removed but reinstated for SDSU v2.1.2 /
           dz=RT0_p0g*(prs_sdsu(l)-prs_sdsu(l1))
!! 06.03.17 end

!! gas amount
           do m=1,kmol
!! 1.d-1=1.d-6*1.d5
! / removed for SDSU /
!              amtpb(m,ll)=amtpb(m,ll)+0.5d0*(cng(l,m)+cng(l1,m))*dz*1.d-1
! / added for SDSU /
              amtpb(m,ll)=amtpb(m,ll)+0.5d0*(cng_sdsu(l,m)+cng_sdsu(l1,m)) &
                                           *dz*1.d-1
           enddo
           amtpb(kmol1,ll)=amtpb(1,ll)**2/(amtpb(1,ll)+dz*1.d5)

! / removed for SDSU /
!! relative humidity (95.12.30)
!           ind=2
!           ppmv=cng(l,1); p1=prs(l); t1=tmp(l)
!           call wvcal(ind,p1,t1,rh1,ppmv,gm3,e,gm3s,es)
!           ppmv=cng(l1,1); p2=prs(l1); t2=tmp(l1)
!           call wvcal(ind,p2,t2,rh2,ppmv,gm3,e,gm3s,es)
! / added for SDSU /
           rh1=rhm_sdsu(l) ; rh2=rhm_sdsu(l1)
!
           rhl=rhl+(rh1+rh2)*0.5d0

!! total dry concentration
! / removed for SDSU /
!           do ipoly=1,npoly
!              do j=1,ncomp(ipoly)
!                 m=mptc(ipoly,j)
!                 cnprf1(ll,ipoly,j)=cnprf1(ll,ipoly,j) &
!                      +(cnp(l,m)+cnp(l1,m))*0.5d0*dzcm
!              enddo
!           enddo
        enddo
!! merging end
!
! / removed for SDSU /
!        do ipoly=1,npoly
!           do j=1,ncomp(ipoly)
!              cnml(ipoly,j)=cnml(ipoly,j)+cnprf1(ll,ipoly,j)
!           enddo
!        enddo
!
! 06.03.17 miho
        pl(ll)=pp/dble(abs(ipb(ll)-ipb(ll+1)))
        tl(ll)=tt/dble(abs(ipb(ll)-ipb(ll+1)))
! 06.03.17 end
        rh(ll)=rhl/dble(abs(ipb(ll)-ipb(ll+1)))
! / added for SDSU/
        dzl(ll)=hgt_sdsu(ipb(ll))-hgt_sdsu(ipb(ll+1))
     enddo
! / added for SDSU /
     Wcw(1:nln) = Wcw_sdsu(nln:1:-1) ; Ncw(1:nln) = Ncw_sdsu(nln:1:-1)
     Wci(1:nln) = Wci_sdsu(nln:1:-1) ; Nci(1:nln) = Nci_sdsu(nln:1:-1)
     Wrr(1:nln) = Wrr_sdsu(nln:1:-1) ; Nrr(1:nln) = Nrr_sdsu(nln:1:-1)
     Wsn(1:nln) = Wsn_sdsu(nln:1:-1) ; Nsn(1:nln) = Nsn_sdsu(nln:1:-1)
     Wgr(1:nln) = Wgr_sdsu(nln:1:-1) ; Ngr(1:nln) = Ngr_sdsu(nln:1:-1)
     Whl(1:nln) = Whl_sdsu(nln:1:-1) ; Nhl(1:nln) = Nhl_sdsu(nln:1:-1)
! / added for SDSU /
     Call watrefind_vir ( wl, crefd )
     scr = real(crefd) ; sci = dimag(crefd)
!
! / removed for SDSU /
! extinction at a scaling wavelength, wlcn
!     volt(1:npoly)=0.d0; cextpn(1:npoly)=0.d0
!     do ipoly=1,npoly
!        do j=1,ncomp(ipoly)
!           m=mptc(ipoly,j)
! dry volume profiles
!           initrf=1
!           do l=1,nln
!              rhl=rh(l)
!
!              if(cnprf1(l,ipoly,j)>0.d0 .and. initrf>0) then
!                 call rfidxb(iuk,wlcn,rhl,m,ispcvp,rfracp,asphr, &
!                      rop,dryap,nawp,awcrp,nv,nwlv,wlv,rfi,scr,sci, &
!                      voldp2,volp2,cextp2,cabsp2,nang,ang,php,err)
!                 if(err.ne.' ') return
!                 if(nawp(m)<=0) initrf=0
!              endif
!
!              if(voldp2*cnml(ipoly,j)>0.d0) then
!                 cnprf1(l,ipoly,j)=cnprf1(l,ipoly,j)/cnml(ipoly,j)*vptc1(ipoly,j)
!                 cextpn(ipoly)=cextpn(ipoly)+cextp2/voldp2*cnprf1(l,ipoly,j)
!                 volt(ipoly)=volt(ipoly)+volp2/voldp2*cnprf1(l,ipoly,j)
!              else
!                 cnprf1(l,ipoly,j)=0.d0
!              endif
!           enddo
!        enddo
!     enddo

! rayleigh phase function
     phsm(1:nang) = 3.d0/16.d0/pi*(1+cos(ang(1:nang)*rad)**2)

! pressure index in each layer
     call sdp2(nln,pl,dp,np)
  endif
! initialization end

! / added for SDSU /
  gtmp=tsk_sdsu

  am0(1:na0)=cos(th0(1:na0)*rad)
  am1u(1:na1)=cos(th1(1:na1)*rad)

  sol=0.d0; wg0=0.d0
  thk0(1:nln,1:10)=0.d0
  flxd0(1:na0,1:ntau)=0.d0
  flxd00(1:na0,1:ntau)=0.d0
  flxu0(1:na0,1:ntau)=0.d0
  ai0(1:na1,1:na0,1:nfi,1:ntau)=0.d0

! wavelength loop start
  do iws=1,nw0
     wla=wl+dw*rx(iws)
     if(wla<0.d0) then
        err='wl+dw*rx hits a negative value'
        return
     endif
     wlb=wl+dw*rx(iws+1)
     wl1=0.5d0*(wla+wlb)
     wgt1 =rf(iws)
     wlc=wl1*1.0d-4

     if(wl1<5.d0) then
        if(wlb==wla) then
           wn=1.d4/wl1
           fsol=sunir(wn)
        else
           call fsoli(wla,wlb,fsol)
           fsol=fsol/(wlb-wla)
        endif
     else
        fsol=0.d0
     endif
     if(isol==0) fsol=0
     
     if(wl1<2.d0) then
        nplk1=0
     else
        nplk1=2
     endif

! polydispersions
!! Rayleigh
     tray=0.00864d0/wl1**(3.916d0+0.074d0*wl1+0.05d0/wl1)
     do l=1,nln
! / removed for SDSU /
!         taur(l)=abs(prs(ipb(l))-prs(ipb(l+1)))/pstd*tray
! / added for SDSU /
        taur(l)=abs(prs_sdsu(ipb(l))-prs_sdsu(ipb(l+1)))/pstd*tray
! / added for SDSUv2.1.0 /
        taur(l)=max(taur(l),1.e-20)
        phsf(1:nang,l)=phsm(1:nang)*taur(l)
     enddo

!! Clouds and Aerosols
     taua(1:nln)=0.d0
     scaa(1:nln)=0.d0
! / added for SDSU /
!     
     cextcw=0.d0;cextci=0.d0;cextrr=0.d0;cextsn=0.d0;cextgr=0.d0;cexthl=0.d0
     cscacw=0.d0;cscaci=0.d0;cscarr=0.d0;cscasn=0.d0;cscagr=0.d0;cscahl=0.d0
     dscscw=0.d0;dscsci=0.d0;dscsrr=0.d0;dscssn=0.d0;dscsgr=0.d0;dscshl=0.d0
     do l = 1, nln
!
        Tavg = 0.5d0 * ( tl(l) + tl(l+1) )
!
        If ( lut ) then
!
           Call mie_lut_vir(Wcw(l),Tavg,Ncw(l),pts_temp_w,ptn_cldw, &
                            cextcw_lut,cscacw_lut,phsfcw_lut, &
                            cextcw(l) ,cscacw(l) ,dscscw(:,l))
!
           Call mie_lut_vir(Wci(l),Tavg,Nci(l),pts_temp_i,ptn_cldi, &
                            cextci_lut,cscaci_lut,phsfci_lut, &
                            cextci(l) ,cscaci(l) ,dscsci(:,l))
!
           Call mie_lut_vir(Wrr(l),Tavg,Nrr(l),pts_temp_w,ptn_rain, &
                            cextrr_lut,cscarr_lut,phsfrr_lut, &
                            cextrr(l) ,cscarr(l) ,dscsrr(:,l))
!
           Call mie_lut_vir(Wsn(l),Tavg,Nsn(l),pts_temp_i,ptn_snow, &
                            cextsn_lut,cscasn_lut,phsfsn_lut, &
                            cextsn(l) ,cscasn(l) ,dscssn(:,l))
!
           Call mie_lut_vir(Wgr(l),Ngr(l),Tavg,pts_temp_i,ptn_grpl, &
                            cextgr_lut,cscagr_lut,phsfgr_lut, &
                            cextgr(l) ,cscagr(l) ,dscsgr(:,l))
!
           Call mie_lut_vir(Whl(l),Nhl(l),Tavg,pts_temp_i,ptn_hail, &
                            cexthl_lut,cscahl_lut,phsfhl_lut, &
                            cexthl(l) ,cscahl(l) ,dscshl(:,l))
!
        else
!
           Call mie_cldw_vir(wl,Tavg,Wcw(l),Ncw(l), &
                             cextcw(l),cscacw(l),dscscw(:,l))
!
           Call mie_cldi_vir(wl,Tavg,Wci(l),Nci(l), &
                             cextci(l),cscaci(l),dscsci(:,l))
!
           Call mie_rain_vir(wl,Tavg,Wrr(l),Nrr(l), &
                             cextrr(l),cscarr(l),dscsrr(:,l))
!
           Call mie_snow_vir(wl,Tavg,Wsn(l),Nsn(l), &
                             cextsn(l),cscasn(l),dscssn(:,l))
!
           Call mie_grpl_vir(wl,Tavg,Wgr(l),Ngr(l), &
                             cextgr(l),cscagr(l),dscsgr(:,l))
!
           Call mie_hail_vir(wl,Tavg,Whl(l),Nhl(l), &
                             cexthl(l),cscahl(l),dscshl(:,l))
!
        End If
!
      end do
!
     taua(1:nln) = cextcw(1:nln) + cextci(1:nln) + cextrr(1:nln) + &
                   cextsn(1:nln) + cextgr(1:nln) + cexthl(1:nln)
     scaa(1:nln) = cscacw(1:nln) + cscaci(1:nln) + cscarr(1:nln) + &
                   cscasn(1:nln) + cscagr(1:nln) + cscahl(1:nln)
     taua(1:nln) = taua(1:nln) * dzl(1:nln)
     scaa(1:nln) = scaa(1:nln) * dzl(1:nln)
     dscs(1:nang,1:nln) = dscscw(1:nang,1:nln) + dscsci(1:nang,1:nln) + &
                          dscsrr(1:nang,1:nln) + dscssn(1:nang,1:nln) + &
                          dscsgr(1:nang,1:nln) + dscshl(1:nang,1:nln)
     do l = 1, nln
        phsf(1:nang,l) = dscs(1:nang,l) * dzl(l) + phsf(1:nang,l)
     end do
                   
! / removed for SDSU /
!      do ipoly=1,npoly
!         taua3(1:nln)=0.d0
!         scaa3(1:nln)=0.d0
!         phsf3(1:nang,1:nln)=0.d0
!
!         do j=1,ncomp(ipoly)
!            m=mptc(ipoly,j)
!            initrf=1
!            do l=1,nln
!               rhl=rh(l)
!               if(cnprf1(l,ipoly,j)>0.d0 .and. initrf>0) then
!                  call rfidxb(iuk,wlc,rhl,m,ispcvp,rfracp,asphr, &
!                       rop,dryap,nawp,awcrp,nv,nwlv,wlv,rfi,scr,sci, &
!                       voldp2,volp2,cextp2,cabsp2,nang,ang,php,err)
!                  if(err.ne.' ') return
!                  if(nawp(m)<=0) initrf=0
!               endif
!               if(cnprf1(l,ipoly,j)>0.d0) then
!                  cext1=cextp2/voldp2*cnprf1(l,ipoly,j)
!                  cabs1=cabsp2/voldp2*cnprf1(l,ipoly,j)
!                  csca1=cext1-cabs1
!                  taua3(l)=taua3(l)+cext1
!                  scaa3(l)=scaa3(l)+csca1
!                  phsf3(1:nang,l)=phsf3(1:nang,l)+php(1:nang)/(cextp2-cabsp2)*csca1
!               else
!                  taua3(l)=0.d0; scaa3(l)=0.d0
!                  phsf3(1:nang,l)=phsf3(1:nang,l)
!               endif
!            enddo
!         enddo
!
!         select case(icn)
!            case(:0); fac=cnpt(ipoly)
!            case(1) ; fac=cnpt(ipoly)/volt(ipoly)
!            case(2) ; fac=cnpt(ipoly)/cextpn(ipoly)
!            case(3:); fac=cnpt(ipoly)/sum(taua3(1:nln)) 
!         end select
!
!         do l=1,nln
!            taua(l)=taua(l)+taua3(l)*fac
!            scaa(l)=scaa(l)+scaa3(l)*fac
!            phsf(1:nang,l)=phsf(1:nang,l)+phsf3(1:nang,l)*fac
!         enddo
!      enddo
         
! gas absorption
! 06.03.17 changed for the k-distribution using hitran 2004.

      nww=int(log10(1.d4/wla)*dintvl+1.d0)-int(log10(1.d4/wlb)*dintvl+1.d0)+1
      call abgask(iug,nln,nch,dintvl,nww,wlb,dp,np,amtpb,tl,tkd,err)
      if(err.ne.'') return 
      call qgausn(gwt,gmu,nch)
      do ik=1,nch
         wgt(ik)=2.d0*gwt(ik)*gmu(ik)
      enddo

      do iww=1,nww
!! nch channnels
         do ik=1,nch
            thk(1:nln)=tkd(1:nln,ik,iww)+taua(1:nln)+taur(1:nln)
            omg(1:nln)=(scaa(1:nln)+taur(1:nln))/thk(1:nln)

            do l=1,nln
! 97. 1.14  20.0->200.0
               taumax=200.d0/(1.04-omg(l))
               thk(l)=min(max(thk(l),1.0d-7),taumax)
            enddo
! indt=1 case
            if(indt==1) then
               lf=0; dpt=0.d0
               do l=1,nln1
                  if(ipbf(l)>0) then
                     lf=lf+1
                     utau(lf)=dpt
                  endif
                  dpt=dpt+thk(l)
               enddo
            endif

! plank function fitting (nplk1=3)
            bgnd=0.d0
            if(nplk1>0) then
! / removed for SDSU /
!               b2=bplnk(wl1,tmp(ipb(1)))
! / added for SDSU /
               b2=bplnk(wl1,tmp_sdsu(ipb(1)))
               do l=1,nln
                  b1=b2
! / removed for SDSU /
!                  b2=bplnk(wl1,tmp(ipb(l+1)))
! / added for SDSU /
                  b2=bplnk(wl1,tmp_sdsu(ipb(l+1)))
                  cplk(1,l)=b1
                  cplk(2,l)=(b2-b1)/thk(l)
               enddo
               if(indg==0) then
                  bgnd=(1-galb)*bplnk(wl1,gtmp)
               else
                  bgnd=bplnk(wl1,gtmp)
               endif
            endif

            call rtrn21(indg,inda,indt,indp,imthd,nda,na1,am1u,na0,am0, &
                 nfi,fi,nln,thk,omg,nlgn1,g,nang,ang,phsf,epsp,epsu,galb,&
                 fsol,nplk1,cplk,bgnd,ntau,utau,scr,sci,flxd,flxu,ai,err)
            if (err /='') return

! 06.03.29 miho
            sol  = sol + fsol*wgt(ik)*wgt1
            wgt2 = wgt(ik)*wgt1
            wg0  = wg0 + wgt2
            thk0(1:nln,1)=thk0(1:nln,1)+thk (1:nln)*wgt2
            thk0(1:nln,2)=thk0(1:nln,2)+taua(1:nln)*wgt2
            thk0(1:nln,3)=thk0(1:nln,3)+taur(1:nln)*wgt2
            sca2(1:nln)=omg(1:nln)*thk(1:nln)*wgt2
            thk0(1:nln,4)=thk0(1:nln,4)+sca2(1:nln)
            thk0(1:nln,5)=thk0(1:nln,5)+g(2,1:nln)*sca2(1:nln)
            thk0(1:nln,6)=thk0(1:nln,6)+g(3,1:nln)*sca2(1:nln)

            lf=0
            thks=0.d0
            do l=1,nln1
               if(l>=2) thks=thks+thk(l-1)
               if(ipbf(l)>0) then
                  lf=lf+1
                  flxd0(1:na0,lf)=flxd0(1:na0,lf)+flxd(1:na0,lf)*wgt2
                  flxd00(1:na0,lf)=flxd00(1:na0,lf) &
                       +fsol*am0(1:na0)*exp(-thks/am0(1:na0))*wgt2
                  flxu0(1:na0,lf)=flxu0(1:na0,lf)+flxu(1:na0,lf)*wgt2
                  if(inda/=0) then
                     ai0(1:na1,1:na0,1:nfi,lf) = ai0(1:na1,1:na0,1:nfi,lf) &
                          + ai(1:na1,1:na0,1:nfi,lf)*wgt2
                  endif
               endif
            enddo
         enddo  ! nch-loop end
      enddo     ! nww-loop end
   enddo        ! iws-loop end

   sol=sol/wg0
   thk0(1:nln,1)=thk0(1:nln,1)/wg0
   thk0(1:nln,2)=thk0(1:nln,2)/wg0
   thk0(1:nln,3)=thk0(1:nln,3)/wg0
   thk0(1:nln,4)=thk0(1:nln,4)/wg0/thk0(1:nln,1)
   thk0(1:nln,5)=thk0(1:nln,5)/wg0/thk0(1:nln,1)
   thk0(1:nln,6)=thk0(1:nln,6)/wg0/thk0(1:nln,1)

   lf=0
   do l=1,nln1
      if(ipbf(l)>0) then
         lf=lf+1
         flxd0(1:na0,lf)=flxd0(1:na0,lf)/wg0
         flxd00(1:na0,lf)=flxd00(1:na0,lf)/wg0
         flxu0(1:na0,lf)=flxu0(1:na0,lf)/wg0
         if(inda/=0) then
            ai0(1:na1,1:na0,1:nfi,lf)=ai0(1:na1,1:na0,1:nfi,lf)/wg0
         endif
      endif
   enddo
   return
 end subroutine rstr6b
